"""Functools backports from standard lib."""

from __future__ import annotations

import sys
import types
from collections.abc import Callable
from typing import Any, Generic, TypeVar, overload

_T = TypeVar("_T")


class cached_property(Generic[_T]):  # pylint: disable=invalid-name
    """
    Backport of Python 3.12's cached_property.

    Includes https://github.com/python/cpython/pull/101890/files
    """

    def __init__(self, func: Callable[[Any], _T]) -> None:
        """Initialize."""
        self.func: Callable[[Any], _T] = func
        self.attrname: str | None = None
        self.__doc__ = func.__doc__

    def __set_name__(self, owner: type[Any], name: str) -> None:
        """Set name."""
        if self.attrname is None:
            self.attrname = name
        elif name != self.attrname:
            raise TypeError(
                "Cannot assign the same cached_property to two different names "
                f"({self.attrname!r} and {name!r})."
            )

    @overload
    def __get__(self, instance: None, owner: type[Any] | None = None) -> Any: ...

    @overload
    def __get__(self, instance: Any, owner: type[Any] | None = None) -> _T: ...

    def __get__(self, instance: Any | None, owner: type[Any] | None = None) -> _T | Any:
        """Get."""
        if instance is None:
            return self
        if self.attrname is None:
            raise TypeError(
                "Cannot use cached_property instance without "
                "calling __set_name__ on it."
            )
        try:
            cache = instance.__dict__
        # not all objects have __dict__ (e.g. class defines slots)
        except AttributeError:
            msg = (
                f"No '__dict__' attribute on {type(instance).__name__!r} "
                f"instance to cache {self.attrname!r} property."
            )
            raise TypeError(msg) from None
        val = self.func(instance)
        try:
            cache[self.attrname] = val
        except TypeError:
            msg = (
                f"The '__dict__' attribute on {type(instance).__name__!r} instance "
                f"does not support item assignment "
                f"for caching {self.attrname!r} property."
            )
            raise TypeError(msg) from None
        return val

    if sys.version_info >= (3, 9):
        __class_getitem__ = classmethod(types.GenericAlias)  # type: ignore
    else:

        def __class_getitem__(
            cls: type[cached_property],
            cls_item: Any,
        ) -> type[cached_property]:
            """Get item."""
            return cls



"""Base implementation."""

import sys
from functools import lru_cache
from ipaddress import AddressValueError, IPv4Address, IPv6Address, NetmaskValueError
from typing import Any, Optional, Union

from .backports.functools import cached_property

if sys.version_info < (3, 9):
    cache = lru_cache(maxsize=None)
else:
    from functools import cache


class CachedIPv4Address(IPv4Address):

    def __init__(self, address: Any) -> None:
        super().__init__(address)
        self.__hash__ = cache(lambda: IPv4Address.__hash__(self))  # type: ignore[method-assign]
        self.__int__ = cache(lambda: IPv4Address.__int__(self))  # type: ignore[method-assign]

    def __str__(self) -> str:
        """Return the string representation of the IPv4 address."""
        return self._str

    @cached_property
    def _str(self) -> str:
        """Return the string representation of the IPv4 address."""
        return super().__str__()

    @cached_property
    def is_link_local(self) -> bool:  # type: ignore[override]
        """Return True if this is a link-local address."""
        return super().is_link_local

    @cached_property
    def is_unspecified(self) -> bool:  # type: ignore[override]
        """Return True if this is an unspecified address."""
        return super().is_unspecified

    @cached_property
    def is_loopback(self) -> bool:  # type: ignore[override]
        """Return True if this is a loopback address."""
        return super().is_loopback

    @cached_property
    def is_multicast(self) -> bool:  # type: ignore[override]
        """Return True if this is a multicast address."""
        return super().is_multicast

    @cached_property
    def reverse_pointer(self) -> str:  # type: ignore[override]
        """Return the reverse DNS pointer name for the IPv4 address."""
        return super().reverse_pointer


class CachedIPv6Address(IPv6Address):

    def __init__(self, address: Any) -> None:
        super().__init__(address)
        self.__hash__ = cache(lambda: IPv6Address.__hash__(self))  # type: ignore[method-assign]
        self.__int__ = cache(lambda: IPv6Address.__int__(self))  # type: ignore[method-assign]

    def __str__(self) -> str:
        """Return the string representation of the IPv6 address."""
        return self._str

    @cached_property
    def _str(self) -> str:
        """Return the string representation of the IPv6 address."""
        return super().__str__()

    @cached_property
    def is_link_local(self) -> bool:  # type: ignore[override]
        """Return True if this is a link-local address."""
        return super().is_link_local

    @cached_property
    def is_unspecified(self) -> bool:  # type: ignore[override]
        """Return True if this is an unspecified address."""
        return super().is_unspecified

    @cached_property
    def is_loopback(self) -> bool:  # type: ignore[override]
        """Return True if this is a loopback address."""
        return super().is_loopback

    @cached_property
    def is_multicast(self) -> bool:  # type: ignore[override]
        """Return True if this is a multicast address."""
        return super().is_multicast

    @cached_property
    def reverse_pointer(self) -> str:  # type: ignore[override]
        """Return the reverse DNS pointer name for the IPv6 address."""
        return super().reverse_pointer


@lru_cache(maxsize=535)
def _cached_ip_addresses(
    address: Union[str, bytes, int]
) -> Optional[Union[IPv4Address, IPv6Address]]:
    """Cache IP addresses."""
    try:
        return CachedIPv4Address(address)
    except (AddressValueError, NetmaskValueError):
        pass

    try:
        return CachedIPv6Address(address)
    except (AddressValueError, NetmaskValueError):
        return None


cached_ip_addresses_wrapper = _cached_ip_addresses
cached_ip_addresses = cached_ip_addresses_wrapper

__all__ = ("cached_ip_addresses",)



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['cached_ipaddress', 'cached_ipaddress.backports']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'cached-ipaddress',
    'version': '0.5.0',
    'description': 'Cache construction of ipaddress objects',
    'long_description': '# cached-ipaddress\n\n<p align="center">\n  <a href="https://github.com/bdraco/cached-ipaddress/actions/workflows/ci.yml?query=branch%3Amain">\n    <img src="https://img.shields.io/github/actions/workflow/status/bdraco/cached-ipaddress/ci.yml?branch=main&label=CI&logo=github&style=flat-square" alt="CI Status" >\n  </a>\n  <a href="https://codecov.io/gh/bdraco/cached-ipaddress">\n    <img src="https://img.shields.io/codecov/c/github/bdraco/cached-ipaddress.svg?logo=codecov&logoColor=fff&style=flat-square" alt="Test coverage percentage">\n  </a>\n</p>\n<p align="center">\n  <a href="https://python-poetry.org/">\n    <img src="https://img.shields.io/badge/packaging-poetry-299bd7?style=flat-square&logo=data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAA4AAAASCAYAAABrXO8xAAAACXBIWXMAAAsTAAALEwEAmpwYAAAAAXNSR0IArs4c6QAAAARnQU1BAACxjwv8YQUAAAJJSURBVHgBfZLPa1NBEMe/s7tNXoxW1KJQKaUHkXhQvHgW6UHQQ09CBS/6V3hKc/AP8CqCrUcpmop3Cx48eDB4yEECjVQrlZb80CRN8t6OM/teagVxYZi38+Yz853dJbzoMV3MM8cJUcLMSUKIE8AzQ2PieZzFxEJOHMOgMQQ+dUgSAckNXhapU/NMhDSWLs1B24A8sO1xrN4NECkcAC9ASkiIJc6k5TRiUDPhnyMMdhKc+Zx19l6SgyeW76BEONY9exVQMzKExGKwwPsCzza7KGSSWRWEQhyEaDXp6ZHEr416ygbiKYOd7TEWvvcQIeusHYMJGhTwF9y7sGnSwaWyFAiyoxzqW0PM/RjghPxF2pWReAowTEXnDh0xgcLs8l2YQmOrj3N7ByiqEoH0cARs4u78WgAVkoEDIDoOi3AkcLOHU60RIg5wC4ZuTC7FaHKQm8Hq1fQuSOBvX/sodmNJSB5geaF5CPIkUeecdMxieoRO5jz9bheL6/tXjrwCyX/UYBUcjCaWHljx1xiX6z9xEjkYAzbGVnB8pvLmyXm9ep+W8CmsSHQQY77Zx1zboxAV0w7ybMhQmfqdmmw3nEp1I0Z+FGO6M8LZdoyZnuzzBdjISicKRnpxzI9fPb+0oYXsNdyi+d3h9bm9MWYHFtPeIZfLwzmFDKy1ai3p+PDls1Llz4yyFpferxjnyjJDSEy9CaCx5m2cJPerq6Xm34eTrZt3PqxYO1XOwDYZrFlH1fWnpU38Y9HRze3lj0vOujZcXKuuXm3jP+s3KbZVra7y2EAAAAAASUVORK5CYII=" alt="Poetry">\n  </a>\n  <a href="https://github.com/ambv/black">\n    <img src="https://img.shields.io/badge/code%20style-black-000000.svg?style=flat-square" alt="black">\n  </a>\n  <a href="https://github.com/pre-commit/pre-commit">\n    <img src="https://img.shields.io/badge/pre--commit-enabled-brightgreen?logo=pre-commit&logoColor=white&style=flat-square" alt="pre-commit">\n  </a>\n</p>\n<p align="center">\n  <a href="https://pypi.org/project/cached-ipaddress/">\n    <img src="https://img.shields.io/pypi/v/cached-ipaddress.svg?logo=python&logoColor=fff&style=flat-square" alt="PyPI Version">\n  </a>\n  <img src="https://img.shields.io/pypi/pyversions/cached-ipaddress.svg?style=flat-square&logo=python&amp;logoColor=fff" alt="Supported Python versions">\n  <img src="https://img.shields.io/pypi/l/cached-ipaddress.svg?style=flat-square" alt="License">\n</p>\n\n---\n\n**Source Code**: <a href="https://github.com/bdraco/cached-ipaddress" target="_blank">https://github.com/bdraco/cached-ipaddress </a>\n\n---\n\nCache construction of ipaddress objects\n\n## Design\n\nThis module keeps a cache of IPAddress objects and caches the properties on them.\n\nIt it useful when you frequently see the same ip addresses over and over and\ndo not want to pay the overhead of constructing IPAddress objects over and over\nor checking their properties.\n\n## Installation\n\nInstall this via pip (or your favourite package manager):\n\n`pip install cached-ipaddress`\n\n## Usage\n\n```python\nfrom cached_ipaddress import cached_ip_addresses\n\nip = cached_ip_addresses("127.0.0.1")\nassert ip.is_loopback is False\n\ninvalid = cached_ip_addresses("invalid")\nassert invalid is None\n```\n\n## Contributors ✨\n\nThanks goes to these wonderful people ([emoji key](https://allcontributors.org/docs/en/emoji-key)):\n\n<!-- prettier-ignore-start -->\n<!-- ALL-CONTRIBUTORS-LIST:START - Do not remove or modify this section -->\n<!-- markdownlint-disable -->\n<!-- markdownlint-enable -->\n<!-- ALL-CONTRIBUTORS-LIST:END -->\n<!-- prettier-ignore-end -->\n\nThis project follows the [all-contributors](https://github.com/all-contributors/all-contributors) specification. Contributions of any kind welcome!\n\n## Credits\n\nThis package was created with\n[Copier](https://copier.readthedocs.io/) and the\n[browniebroke/pypackage-template](https://github.com/browniebroke/pypackage-template)\nproject template.\n',
    'author': 'J. Nick Koston',
    'author_email': 'nick@koston.org',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/bdraco/cached-ipaddress',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.8,<4.0',
}
from build_ext import *
build(setup_kwargs)

setup(**setup_kwargs)
